@@ -8,22 +8,17 @@ TAG="$3"
 USER="$4"
 PASSWORD="$5"
 ARCHITECTURE="$6"
-DOCKER_PLATFORM="$7"
 
 cp "$DEB_FILE" "$DEB_DEBUG_FILE" .
 
-if [ "${ARCHITECTURE}" = "armhf" ]; then
-  DOCKERFILE=.github/docker/Dockerfile-armhf.production
-else
-  DOCKERFILE=.github/docker/Dockerfile.production
-fi
+DOCKERFILE=.github/docker/Dockerfile.production
 
 docker login -u "$USER" -p "$PASSWORD"
 
-docker buildx build \
+docker build \
   --pull \
-  --platform "${DOCKER_PLATFORM}" \
   --no-cache \
+  --provenance false \
   --build-arg "DEB_FILE=$DEB_FILE" \
   --build-arg "DEB_DEBUG_FILE=$DEB_DEBUG_FILE" \
   --file "${DOCKERFILE}" \
