@@ -3,7 +3,6 @@
 set -e
 
 CPU_CORES="$1"
-PLATFORM="$2"
 
 export CPU_CORES
 
@@ -44,11 +43,18 @@ cd /tmp/liquidsoap-full/liquidsoap
 
 ./.github/scripts/checkout-deps.sh
 
+git clone https://github.com/savonet/ocaml-mem_usage.git
+cd ocaml-mem_usage
+opam install -y .
+cd ..
+
 opam update
-opam install -y magic-mime
+opam remove -y jemalloc
+opam install -y tls.1.0.2 ca-certs mirage-crypto-rng cstruct saturn_lockfree.0.4.1 ppx_hash memtrace
 
 cd /tmp/liquidsoap-full
 
+# TODO: Remove gstreamer from liquidsoap-full
 sed -e 's@ocaml-gstreamer@#ocaml-gstreamer@' -i PACKAGES
 
 export PKG_CONFIG_PATH=/usr/share/pkgconfig/pkgconfig
@@ -82,10 +88,6 @@ dune build --profile=release
 
 echo "::endgroup::"
 
-if [ "${PLATFORM}" = "armhf" ]; then
-  exit 0
-fi
-
 echo "::group::Print build config"
 
 dune exec -- liquidsoap --build-config
