@@ -8,8 +8,7 @@ let () =
       ignore
         (Lang.add_builtin_base ~category:`Configuration
            ~descr:("Build-time configuration for " ^ name)
-           ~base:liquidsoap_build_config_optionals name
-           Lang.(Ground (Ground.Bool value))
+           ~base:liquidsoap_build_config_optionals name (`Bool value)
            Lang.bool_t))
     [
       ("alsa", Alsa_option.enabled);
@@ -24,7 +23,6 @@ let () =
       ("frei0r", Frei0r_option.enabled);
       ("gd", Gd_option.enabled);
       ("graphics", Graphics_option.enabled);
-      ("gstreamer", Gstreamer_option.enabled);
       ("inotify", Inotify_option.enabled);
       ("imagelib", Imagelib_option.enabled);
       ("irc", Irc_option.enabled);
@@ -34,7 +32,6 @@ let () =
       ("lilv", Lilv_option.enabled);
       ("lo", Lo_option.enabled);
       ("mad", Mad_option.enabled);
-      ("mem_usage", Mem_usage_option.enabled);
       ("memtrace", Memtrace_option.enabled);
       ("ogg", Ogg_option.enabled);
       ("opus", Opus_option.enabled);
@@ -52,7 +49,6 @@ let () =
       ("srt", Srt_option.enabled);
       ("ssl", Ssl_option.enabled);
       ("tls", Tls_option.enabled);
-      ("taglib", Taglib_option.enabled);
       ("theora", Theora_option.enabled);
       ("vorbis", Vorbis_option.enabled);
       ("winsvc", Winsvc_option.enabled);
