@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -31,6 +31,22 @@ let _ =
   Lang.add_builtin ~base:decoder_metadata "add" ~category:`Liquidsoap
     ~descr:"Register an external file metadata decoder."
     [
+      ( "priority",
+        Lang.getter_t Lang.int_t,
+        Some (Lang.int 1),
+        Some "Resolver's priority." );
+      ( "mime_types",
+        Lang.nullable_t (Lang.list_t Lang.string_t),
+        Some Lang.null,
+        Some
+          "Decode files that match the mime types in this list. Accept any \
+           file if `null`." );
+      ( "file_extensions",
+        Lang.nullable_t (Lang.list_t Lang.string_t),
+        Some Lang.null,
+        Some
+          "Decode files that have the file extensions in this list. Accept any \
+           file if `null`." );
       ("", Lang.string_t, None, Some "Format/resolver's name.");
       ( "",
         resolver_t,
@@ -43,10 +59,25 @@ let _ =
     (fun p ->
       let format = Lang.to_string (Lang.assoc "" 1 p) in
       let f = Lang.assoc "" 2 p in
-      let resolver ~metadata ~extension:_ ~mime:_ name =
+      let mimes =
+        Lang.to_valued_option
+          (fun v -> List.map Lang.to_string (Lang.to_list v))
+          (List.assoc "mime_types" p)
+      in
+      let extensions =
+        Lang.to_valued_option
+          (fun v -> List.map Lang.to_string (Lang.to_list v))
+          (List.assoc "file_extensions" p)
+      in
+      let log = Log.make ["decoder"; "metadata"] in
+      let priority = Lang.to_int_getter (List.assoc "priority" p) in
+      let resolver ~metadata ~extension ~mime fname =
+        if
+          not (Decoder.test_file ~log ~extension ~mime ~mimes ~extensions fname)
+        then raise Metadata.Invalid;
         let ret =
           Lang.apply f
-            [("metadata", Lang.metadata metadata); ("", Lang.string name)]
+            [("metadata", Lang.metadata metadata); ("", Lang.string fname)]
         in
         let ret = Lang.to_list ret in
         let ret = List.map Lang.to_product ret in
@@ -55,7 +86,8 @@ let _ =
         in
         ret
       in
-      Plug.register Request.mresolvers format ~doc:"" resolver;
+      Plug.register Request.mresolvers format ~doc:""
+        { Request.priority; resolver };
       Lang.unit)
 
 let add_playlist_parser ~format name (parser : Playlist_parser.parser) =
@@ -127,6 +159,9 @@ let _ =
         { Playlist_parser.strict; Playlist_parser.parser = fn };
       Lang.unit)
 
+let default_static =
+  Lang.eval ~cache:false ~typecheck:false ~stdlib:`Disabled "fun (_) -> false"
+
 let _ =
   let log_p = [("", "", None)] in
   let log_t = Lang.fun_t [(false, "", Lang.string_t)] Lang.unit_t in
@@ -137,7 +172,7 @@ let _ =
         (false, "maxtime", Lang.float_t);
         (false, "", Lang.string_t);
       ]
-      (Lang.list_t Lang.string_t)
+      Lang.(nullable_t string_t)
   in
   Lang.add_builtin ~base:Modules.protocol "add" ~category:`Liquidsoap
     ~descr:"Register a new protocol."
@@ -147,11 +182,12 @@ let _ =
         Some (Lang.bool false),
         Some "if true, file is removed when it is finished." );
       ( "static",
-        Lang.bool_t,
-        Some (Lang.bool false),
+        Lang.fun_t [(false, "", Lang.string_t)] Lang.bool_t,
+        Some default_static,
         Some
-          "if true, then requests can be resolved once and for all. Typically, \
-           static protocols can be used to create infallible sources." );
+          "When given an uri for the protocol, if it returns `true`, then \
+           requests can be resolved once and for all. Typically, static \
+           protocols can be used to create infallible sources." );
       ( "syntax",
         Lang.string_t,
         Some (Lang.string "Undocumented"),
@@ -179,7 +215,8 @@ let _ =
       let name = Lang.to_string (Lang.assoc "" 1 p) in
       let f = Lang.assoc "" 2 p in
       let temporary = Lang.to_bool (List.assoc "temporary" p) in
-      let static = Lang.to_bool (List.assoc "static" p) in
+      let static = List.assoc "static" p in
+      let static s = Lang.to_bool (Lang.apply static [("", Lang.string s)]) in
       let doc = Lang.to_string (List.assoc "doc" p) in
       let syntax = Lang.to_string (List.assoc "syntax" p) in
       Lang.add_protocol ~syntax ~doc ~static name (fun arg ~log timeout ->
@@ -189,7 +226,7 @@ let _ =
                 log (Lang.to_string v);
                 Lang.unit)
           in
-          let l =
+          let ret =
             Lang.apply f
               [
                 ("rlog", log);
@@ -197,7 +234,12 @@ let _ =
                 ("", Lang.string arg);
               ]
           in
-          List.map
+          Option.map
             (fun s -> Request.indicator ~temporary (Lang.to_string s))
-            (Lang.to_list l));
+            (Lang.to_option ret));
       Lang.unit)
+
+let _ =
+  Lang.add_builtin ~base:Modules.protocol "count" ~category:`Liquidsoap
+    ~descr:"Number of registered protocols." [] Lang.int_t (fun _ ->
+      Doc.Protocol.count () |> Lang.int)
