@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
 
  *****************************************************************************)
 
+module Http = Liq_http
+
 module Liq_tls = struct
   type t = {
     read_pending : Buffer.t;
@@ -31,13 +33,23 @@ module Liq_tls = struct
   let () = Mirage_crypto_rng_unix.initialize (module Mirage_crypto_rng.Fortuna)
   let buf_len = 4096
 
+  let string_of_alert_level = function
+    | Tls.Packet.WARNING -> "Warning"
+    | Tls.Packet.FATAL -> "Fatal"
+
+  let string_of_failure error =
+    let level, typ = Tls.Engine.alert_of_failure error in
+    Printf.sprintf "%s error: %s"
+      (string_of_alert_level level)
+      (Tls.Packet.alert_type_to_string typ)
+
   let write_all ~timeout fd data =
-    Tutils.write_all ~timeout fd (Cstruct.to_bytes data)
+    Tutils.write_all ~timeout fd (Bytes.unsafe_of_string data)
 
   let read ~timeout h len =
     Tutils.wait_for (`Read h.fd) timeout;
     let n = Unix.read h.fd h.buf 0 (min len buf_len) in
-    Cstruct.of_bytes ~len:n h.buf
+    Bytes.sub_string h.buf 0 n
 
   let read_pending h = function
     | None -> ()
@@ -51,30 +63,21 @@ module Liq_tls = struct
     let rec f () =
       if Tls.Engine.handshake_in_progress h.state then (
         match Tls.Engine.handle_tls h.state (read ~timeout h buf_len) with
-          | Ok (`Eof, _, _) ->
+          | Ok (_, Some `Eof, _, _) ->
               Runtime_error.raise ~pos:[]
                 ~message:"Connection closed while negotiating TLS handshake!"
                 "tls"
-          | Ok ((`Ok _ as step), `Response response, `Data data)
-          | Ok ((`Alert _ as step), `Response response, `Data data) ->
-              read_pending h data;
+          | Ok (state, None, `Response response, `Data data) ->
+              h.state <- state;
+              read_pending h (Option.map Cstruct.of_string data);
               write_response ~timeout h response;
-              (match step with
-                | `Ok state -> h.state <- state
-                | `Alert alert ->
-                    Runtime_error.raise ~pos:[]
-                      ~message:
-                        (Printf.sprintf "TLS handshake error: %s"
-                           (Tls.Packet.alert_type_to_string alert))
-                      "tls");
               f ()
           | Error (error, `Response response) ->
               write_all ~timeout h.fd response;
               Runtime_error.raise ~pos:[]
                 ~message:
                   (Printf.sprintf "TLS handshake error: %s"
-                     (Tls.Packet.alert_type_to_string
-                        (Tls.Engine.alert_of_failure error)))
+                     (string_of_failure error))
                 "tls")
     in
     f ()
@@ -98,7 +101,7 @@ module Liq_tls = struct
   let write ?timeout h b off len =
     let timeout = Option.value ~default:Harbor_base.conf_timeout#get timeout in
     match
-      Tls.Engine.send_application_data h.state [Cstruct.of_bytes ~off ~len b]
+      Tls.Engine.send_application_data h.state [Bytes.sub_string b off len]
     with
       | None -> len
       | Some (state, data) ->
@@ -124,38 +127,26 @@ module Liq_tls = struct
         match
           Tls.Engine.handle_tls h.state (read ~timeout:read_timeout h len)
         with
-          | Ok (`Eof, _, _) -> 0
-          | Ok (`Alert alert, `Response response, _) ->
-              (match response with
-                | None -> ()
-                | Some r -> write_all ~timeout:write_timeout h.fd r);
-              Runtime_error.raise ~pos:[]
-                ~message:
-                  (Printf.sprintf "TLS read error: %s"
-                     (Tls.Packet.alert_type_to_string alert))
-                "tls"
-          | Ok (`Ok state, `Response response, `Data data) -> (
+          | Ok (state, eof, `Response response, `Data data) -> (
               (match response with
                 | None -> ()
                 | Some r -> write_all ~timeout:write_timeout h.fd r);
               h.state <- state;
-              match data with
-                | None -> f ()
-                | Some data ->
-                    let data_len = Cstruct.length data in
+              match (eof, data) with
+                | Some `Eof, None -> 0
+                | _, None -> f ()
+                | _, Some data ->
+                    let data_len = String.length data in
                     let n = min data_len len in
-                    Cstruct.blit_to_bytes data 0 b off n;
+                    Bytes.blit_string data 0 b off n;
                     if n < data_len then
-                      Buffer.add_string h.read_pending
-                        (Cstruct.to_string data ~off:n ~len:(data_len - n));
+                      Buffer.add_substring h.read_pending data n (data_len - n);
                     n)
           | Error (error, `Response response) ->
               write_all ~timeout:write_timeout h.fd response;
               Runtime_error.raise ~pos:[]
                 ~message:
-                  (Printf.sprintf "TLS read error: %s"
-                     (Tls.Packet.alert_type_to_string
-                        (Tls.Engine.alert_of_failure error)))
+                  (Printf.sprintf "TLS read error: %s" (string_of_failure error))
                 "tls"
       in
       f ())
@@ -190,16 +181,18 @@ let tls_socket ~session transport =
 let server ~read_timeout ~write_timeout ~certificate ~key transport =
   let server =
     try
-      let certificate = Cstruct.of_string (Utils.read_all (certificate ())) in
+      let certificate = Utils.read_all (certificate ()) in
       let certificates =
         Result.get_ok (X509.Certificate.decode_pem_multiple certificate)
       in
       let key =
-        Result.get_ok
-          (X509.Private_key.decode_pem
-             (Cstruct.of_string (Utils.read_all (key ()))))
+        Result.get_ok (X509.Private_key.decode_pem (Utils.read_all (key ())))
       in
-      Tls.Config.server ~certificates:(`Single (certificates, key)) ()
+      match
+        Tls.Config.server ~certificates:(`Single (certificates, key)) ()
+      with
+        | Ok server -> server
+        | Error (`Msg message) -> Runtime_error.raise ~pos:[] ~message "tls"
     with exn ->
       let bt = Printexc.get_raw_backtrace () in
       Lang.raise_as_runtime ~bt ~kind:"tls" exn
@@ -234,7 +227,7 @@ let transport ~read_timeout ~write_timeout ~certificate ~key () =
           let certificates =
             Result.get_ok
               (X509.Certificate.decode_pem_multiple
-                 (Cstruct.of_string (Utils.read_all (certificate ()))))
+                 (Utils.read_all (certificate ())))
           in
           Some
             (X509.Authenticator.chain_of_trust
@@ -249,7 +242,11 @@ let transport ~read_timeout ~write_timeout ~certificate ~key () =
               let r = auth ?ip ~host certs in
               if Result.is_ok r then r else authenticator ?ip ~host certs
       in
-      let client = Tls.Config.client ~authenticator ~peer_name:domain () in
+      let client =
+        match Tls.Config.client ~authenticator ~peer_name:domain () with
+          | Ok client -> client
+          | Error (`Msg message) -> Runtime_error.raise ~pos:[] ~message "tls"
+      in
       let fd = Http.connect ?bind_address ~timeout ?prefer host port in
       let session = Liq_tls.init_client ~timeout ~client fd in
       tls_socket ~session self
