@@ -1,6 +1,6 @@
 (*****************************************************************************
 
-    Liquidsoap, a programmable audio stream generator.
+    Liquidsoap, a programmable stream generator.
     Copyright 2003-2016 Savonet team
 
     This program is free software; you can redistribute it and/or modify
@@ -23,8 +23,7 @@
 open Harbor_base
 module Monad = Duppy.Monad
 module Type = Liquidsoap_lang.Type
-module Regexp = Liquidsoap_lang.Regexp
-module Http_base = Http
+module Http = Liq_http
 
 let ( let* ) = Duppy.Monad.bind
 
@@ -161,7 +160,7 @@ module type T = sig
         socket ->
         unit
 
-      method virtual insert_metadata : (string, string) Hashtbl.t -> unit
+      method virtual insert_metadata : Frame.metadata -> unit
 
       method virtual login :
         string * (socket:socket -> string -> string -> bool)
@@ -224,7 +223,7 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
             socket ->
             unit
 
-      method virtual insert_metadata : (string, string) Hashtbl.t -> unit
+      method virtual insert_metadata : Frame.metadata -> unit
 
       method virtual login
           : string * (socket:socket -> string -> string -> bool)
@@ -362,11 +361,9 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
   let websocket_error n msg = Websocket.to_string (`Close (Some (n, msg)))
 
   let parse_icy_request_line ~port h r =
-    let auth_data = Pcre.split ~pat:":" r in
-    let requested_user, password =
-      match auth_data with
-        | user :: password :: _ -> (user, password)
-        | _ -> ("", r)
+    let { Liq_http.user = requested_user; password } =
+      try Liq_http.parse_auth r
+      with Not_found -> { Liq_http.user = ""; password = r }
     in
     let* s =
       try Duppy.Monad.return (find_source "/" (port - 1))
@@ -388,7 +385,7 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
 
   let parse_http_request_line r =
     try
-      let data = Pcre.split ~rex:(Pcre.regexp "[ \t]+") r in
+      let data = Re.Pcre.split ~rex:(Re.Pcre.regexp "[ \t]+") r in
       let protocol = verb_or_source_of_string (List.nth data 0) in
       Duppy.Monad.return
         ( protocol,
@@ -410,9 +407,9 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
   let parse_headers headers =
     let split_header h l =
       try
-        let rex = Pcre.regexp "([^:\\r\\n]+):\\s*([^\\r\\n]+)" in
-        let sub = Pcre.exec ~rex h in
-        (Pcre.get_substring sub 1, Pcre.get_substring sub 2) :: l
+        let rex = Re.Pcre.regexp "([^:\\r\\n]+):\\s*([^\\r\\n]+)" in
+        let sub = Re.Pcre.exec ~rex h in
+        (Re.Pcre.get_substring sub 1, Re.Pcre.get_substring sub 2) :: l
       with Not_found -> l
     in
     let f x = String.uppercase_ascii x in
@@ -450,13 +447,13 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
         try
           (* HTTP authentication *)
           let auth = assoc_uppercase "AUTHORIZATION" headers in
-          let data = Pcre.split ~rex:(Pcre.regexp "[ \t]+") auth in
+          let data = Re.Pcre.split ~rex:(Re.Pcre.regexp "[ \t]+") auth in
           match data with
-            | "Basic" :: x :: _ -> (
-                let auth_data = Pcre.split ~pat:":" (Lang_string.decode64 x) in
-                match auth_data with
-                  | x :: y :: _ -> (x, y)
-                  | _ -> raise Not_found)
+            | "Basic" :: x :: _ ->
+                let { Liq_http.user; password } =
+                  Liq_http.parse_auth (Lang_string.decode64 x)
+                in
+                (user, password)
             | _ -> raise Not_found
         with Not_found -> (
           match query with
@@ -658,13 +655,7 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
                   log#debug "Metadata packet: %s\n%!" s;
                   let data = Option.get data in
                   let m = List.map (fun (l, v) -> (l, json_string_of v)) data in
-                  let m =
-                    let ans = Hashtbl.create (List.length m) in
-                    (* TODO: convert charset *)
-                    let g x = x in
-                    List.iter (fun (l, v) -> Hashtbl.add ans (g l) (g v)) m;
-                    ans
-                  in
+                  let m = Frame.Metadata.from_list m in
                   source#insert_metadata m;
                   raise Retry
               | _ -> raise Retry)
@@ -715,7 +706,7 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
        if
          assoc_uppercase "CONTENT-TYPE" headers
          = "application/x-www-form-urlencoded"
-       then Hashtbl.iter (Hashtbl.add args) (Http.args_split data)
+       then Hashtbl.iter (Hashtbl.replace args) (Http.args_split data)
      with Not_found -> ());
     match mode with
       | "updinfo" ->
@@ -762,12 +753,9 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
                 | _ -> x
             in
             let g x = Charset.convert ?source:in_enc x in
-            Hashtbl.add m (g x) (g y);
-            m
-          in
-          let args =
-            Hashtbl.fold f args (Hashtbl.create (Hashtbl.length args))
+            Frame.Metadata.add (g x) (g y) m
           in
+          let args = Hashtbl.fold f args Frame.Metadata.empty in
           s#insert_metadata args;
           simple_reply
             (Printf.sprintf
@@ -775,11 +763,11 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
       | _ -> ans_500 uri
 
   let handle_http_request ~hmethod ~hprotocol ~port h uri headers =
-    let rex = Pcre.regexp "^(.+)\\?(.+)$" in
+    let rex = Re.Pcre.regexp "^(.+)\\?(.+)$" in
     let base_uri, args =
       try
-        let sub = Pcre.exec ~rex uri in
-        (Pcre.get_substring sub 1, Pcre.get_substring sub 2)
+        let sub = Re.Pcre.exec ~rex uri in
+        (Re.Pcre.get_substring sub 1, Re.Pcre.get_substring sub 2)
       with Not_found -> (uri, "")
     in
     let smethod = string_of_verb hmethod in
@@ -797,11 +785,25 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
 
     (* First, try with a registered handler. *)
     let { handler; _ } = find_handler port in
-    let f (verb, rex, handler) =
-      if (verb :> verb) = hmethod && Lang.Regexp.test rex base_uri then (
-        let { Lang.Regexp.groups } = Lang.Regexp.exec rex base_uri in
+    let f (verb, regex, handler) =
+      let rex = regex.Liquidsoap_lang.Builtins_regexp.regexp in
+      let sub =
+        Lazy.from_fun (fun () ->
+            try Some (Re.Pcre.exec ~rex base_uri) with _ -> None)
+      in
+      if (verb :> verb) = hmethod && Lazy.force sub <> None then (
+        let sub = Option.get (Lazy.force sub) in
+        let groups =
+          List.fold_left
+            (fun groups name ->
+              try (name, Re.Pcre.get_named_substring rex name sub) :: groups
+              with Not_found -> groups)
+            []
+            (Array.to_list (Re.Pcre.names rex))
+        in
         log#info "Found handler '%s %s' on port %d%s." smethod
-          (Lang.descr_of_regexp rex) port
+          (Lang.descr_of_regexp regex)
+          port
           (match groups with
             | [] -> ""
             | groups ->
@@ -914,7 +916,7 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
             | false -> Duppy.Io.Split "[\r]?\n[\r]?\n")
         h
     in
-    let lines = Pcre.split ~rex:(Pcre.regexp "[\r]?\n") s in
+    let lines = Re.Pcre.split ~rex:(Re.Pcre.regexp "[\r]?\n") s in
     let* hmethod, huri, hprotocol =
       let s = List.hd lines in
       if icy then parse_icy_request_line ~port h s
@@ -996,7 +998,7 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
               ~priority:`Non_blocking ~marker:(Duppy.Io.Split "[\r]?\n[\r]?\n")
               h
           in
-          let lines = Pcre.split ~rex:(Pcre.regexp "[\r]?\n") s in
+          let lines = Re.Pcre.split ~rex:(Re.Pcre.regexp "[\r]?\n") s in
           let headers = parse_headers lines in
           handle_source_request ~port ~auth:true ~smethod:`Shout hprotocol h
             huri headers
@@ -1125,7 +1127,7 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
   let get_handler ~pos ~transport ~icy port =
     try
       let { handler; fds; transport = t } = Hashtbl.find opened_ports port in
-      if transport != t then
+      if transport#name <> t#name then
         Lang.raise_error ~pos
           ~message:"Port is already opened with a different transport" "http";
       (* If we have only one socket and icy=true,
@@ -1143,7 +1145,7 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
         if icy then open_port ~transport ~icy (port + 1) :: fds else fds
       in
       let handler = { sources = Hashtbl.create 1; http = Atomic.make [] } in
-      Hashtbl.add opened_ports port { handler; fds; transport };
+      Hashtbl.replace opened_ports port { handler; fds; transport };
       handler
 
   (* Add sources... This is tied up to sources lifecycle so
@@ -1157,7 +1159,7 @@ module Make (T : Transport_t) : T with type socket = T.socket = struct
       handler.sources
     in
     log#important "Adding mountpoint '%s' on port %i" mountpoint port;
-    Hashtbl.add sources mountpoint source
+    Hashtbl.replace sources mountpoint source
 
   (* Remove source. *)
   let remove_source ~port ~mountpoint () =
