@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,6 +22,8 @@
 
 exception Not_connected
 
+module Http = Liq_http
+
 module Metadata = struct
   include Map.Make (struct
     type t = string
@@ -61,27 +63,41 @@ let () =
       Atomic.set shutdown true)
 
 class input ?(name = "input.ffmpeg") ~autostart ~self_sync ~poll_delay ~debug
-  ~clock_safe ~max_buffer ~on_error ~on_stop ~on_start ~on_connect
-  ~metadata_filter ~on_disconnect ~new_track_on_metadata ?format ~opts ~trim_url
-  url =
+  ~max_buffer ~on_error ~on_stop ~on_start ~on_connect ~metadata_filter
+  ~on_disconnect ~new_track_on_metadata ?format ~opts ~trim_url url =
   let max_length = Some (Frame.main_of_seconds max_buffer) in
   object (self)
     inherit
       Start_stop.active_source
-        ~name ~fallible:true ~clock_safe ~on_start ~on_stop ~autostart () as super
+        ~name ~fallible:true ~on_start ~on_stop ~autostart () as super
 
     val connect_task = Atomic.make None
-    val container = Atomic.make None
     method seek_source = (self :> Source.source)
     method remaining = -1
     method abort_track = Generator.add_track_mark self#buffer
-    method private is_connected = Atomic.get container <> None
-    method! is_ready ?frame () = super#is_ready ?frame () && self#is_connected
+
+    val source_status
+        : [ `Stopped
+          | `Starting
+          | `Polling
+          | `Connected of string * container
+          | `Stopping ]
+          Atomic.t =
+      Atomic.make `Stopped
+
+    method source_status = Atomic.get source_status
+
+    method private is_connected =
+      match Atomic.get source_status with `Connected _ -> true | _ -> false
+
+    method can_generate_frame = super#started && self#is_connected
 
     method private get_self_sync =
       match self_sync () with Some v -> v | None -> false
 
-    method self_sync = (`Dynamic, self#get_self_sync && self#is_connected)
+    method self_sync =
+      (`Dynamic, self#source_sync (self#get_self_sync && self#is_connected))
+
     method private start = self#connect
     method private stop = self#disconnect
     val mutable url = url
@@ -93,18 +109,10 @@ class input ?(name = "input.ffmpeg") ~autostart ~self_sync ~poll_delay ~debug
     method set_url u = url <- u
     method buffer_length = Frame.seconds_of_audio (Generator.length self#buffer)
 
-    val source_status
-        : [ `Stopped | `Starting | `Polling | `Connected of string | `Stopping ]
-          Atomic.t =
-      Atomic.make `Stopped
-
-    method source_status = Atomic.get source_status
-
     method private connect_task () =
       Generator.set_max_length self#buffer max_length;
       try
         if self#source_status = `Stopping then raise Stopped;
-        assert (Atomic.get container = None);
         assert (self#source_status = `Starting);
         Atomic.set source_status `Polling;
         let opts = Hashtbl.copy opts in
@@ -165,9 +173,9 @@ class input ?(name = "input.ffmpeg") ~autostart ~self_sync ~poll_delay ~debug
           else []
         in
         on_connect input;
-        Atomic.set container
-          (Some { input; decoder; buffer; get_metadata; closed });
-        Atomic.set source_status (`Connected url);
+        Generator.add_track_mark self#buffer;
+        let container = { input; decoder; buffer; get_metadata; closed } in
+        Atomic.set source_status (`Connected (url, container));
         -1.
       with
         | Stopped ->
@@ -184,24 +192,32 @@ class input ?(name = "input.ffmpeg") ~autostart ~self_sync ~poll_delay ~debug
             poll_delay
 
     method private connect =
-      if Atomic.get container = None then (
-        assert (
-          match self#source_status with `Connected _ -> false | _ -> true);
-        Atomic.set source_status `Starting;
-        match Atomic.get connect_task with
-          | Some t -> Duppy.Async.wake_up t
-          | None ->
-              let t =
-                Duppy.Async.add ~priority:`Blocking Tutils.scheduler
-                  self#connect_task
-              in
-              Atomic.set connect_task (Some t);
-              Duppy.Async.wake_up t)
+      match self#source_status with
+        | `Starting | `Polling | `Connected _ -> ()
+        | `Stopping | `Stopped -> (
+            Atomic.set source_status `Starting;
+            match Atomic.get connect_task with
+              | Some t -> Duppy.Async.wake_up t
+              | None ->
+                  let t =
+                    Duppy.Async.add ~priority:`Blocking Tutils.scheduler
+                      self#connect_task
+                  in
+                  Atomic.set connect_task (Some t);
+                  Duppy.Async.wake_up t)
 
     method private disconnect =
-      (match Atomic.exchange container None with
-        | None -> ()
-        | Some { input; closed } ->
+      let stop_task () =
+        match Atomic.get connect_task with
+          | None -> ()
+          | Some t ->
+              Atomic.set source_status `Stopping;
+              Duppy.Async.wake_up t
+      in
+      match self#source_status with
+        | `Stopping | `Stopped -> ()
+        | `Polling | `Starting -> stop_task ()
+        | `Connected (_, { input; closed }) ->
             Atomic.set closed true;
             self#mutexify
               (fun () ->
@@ -212,27 +228,20 @@ class input ?(name = "input.ffmpeg") ~autostart ~self_sync ~poll_delay ~debug
                     (Printf.sprintf "Error while disconnecting: %s"
                        (Printexc.to_string exn)))
               ();
-            on_disconnect ());
-      (* Make sure the polling task stops as well. *)
-      ignore
-        (Option.map
-           (fun t ->
-             Atomic.set source_status `Stopping;
-             Duppy.Async.wake_up t)
-           (Atomic.get connect_task))
+            on_disconnect ();
+            stop_task ()
 
     method private reconnect =
       self#disconnect;
       self#connect
 
     method private get_connected_container =
-      match Atomic.get container with
-        | None -> raise Not_connected
-        | Some c -> c
+      match self#source_status with
+        | `Connected (_, c) -> c
+        | _ -> raise Not_connected
 
-    method private get_frame frame =
-      let pos = Frame.position frame in
-      let breaks = Frame.breaks frame in
+    method private generate_frame =
+      let size = Lazy.force Frame.size in
       try
         let { decoder; buffer; closed } = self#get_connected_container in
         while Generator.length self#buffer < Lazy.force Frame.size do
@@ -242,40 +251,35 @@ class input ?(name = "input.ffmpeg") ~autostart ~self_sync ~poll_delay ~debug
         let { get_metadata } = self#get_connected_container in
         let meta = get_metadata () in
         if meta <> [] then (
-          let m = Hashtbl.create (List.length meta) in
-          List.iter (fun (k, v) -> Hashtbl.add m k v) meta;
-          Generator.add_metadata self#buffer m;
+          Generator.add_metadata self#buffer (Frame.Metadata.from_list meta);
           if new_track_on_metadata then Generator.add_track_mark self#buffer);
-        Generator.fill self#buffer frame;
-        let stop = Frame.position frame in
+        let frame = Generator.slice self#buffer size in
         (* Metadata can be added by the decoder and the demuxer so we filter at the frame level. *)
         let metadata =
           List.fold_left
             (fun metadata (p, m) ->
-              if p < pos || stop < p then (p, m) :: metadata
-              else (
-                let m = metadata_filter m in
-                if 0 < Hashtbl.length m then (p, m) :: metadata else metadata))
+              let m = metadata_filter m in
+              if 0 < Frame.Metadata.cardinal m then (p, m) :: metadata
+              else metadata)
             []
             (Frame.get_all_metadata frame)
         in
-        Frame.set_all_metadata frame metadata
+        Frame.add_all_metadata frame metadata
       with exn ->
         let bt = Printexc.get_raw_backtrace () in
         Utils.log_exception ~log:self#log
           ~bt:(Printexc.raw_backtrace_to_string bt)
           (Printf.sprintf "Feeding failed: %s" (Printexc.to_string exn));
         on_error (Lang.runtime_error_of_exception ~bt ~kind:"ffmpeg" exn);
-        if List.length breaks = List.length (Frame.breaks frame) then
-          Frame.add_break frame pos;
-        self#reconnect
+        self#reconnect;
+        Frame.append (Generator.slice self#buffer size) self#end_of_track
   end
 
 let http_log = Log.make ["input"; "http"]
 
-class http_input ~autostart ~self_sync ~poll_delay ~debug ~clock_safe ~on_error
-  ~max_buffer ~on_connect ~on_disconnect ?format ~opts ~user_agent ~timeout
-  ~metadata_filter ~on_start ~on_stop ~new_track_on_metadata ~trim_url url =
+class http_input ~autostart ~self_sync ~poll_delay ~debug ~on_error ~max_buffer
+  ~on_connect ~on_disconnect ?format ~opts ~user_agent ~timeout ~metadata_filter
+  ~on_start ~on_stop ~new_track_on_metadata ~trim_url url =
   let () =
     Hashtbl.replace opts "icy" (`Int 1);
     Hashtbl.replace opts "user_agent" (`String user_agent);
@@ -290,13 +294,18 @@ class http_input ~autostart ~self_sync ~poll_delay ~debug ~clock_safe ~on_error
           Avutil.Options.get_string ~search_children:true
             ~name:"icy_metadata_headers" (Av.input_obj input)
         in
-        let icy_headers = Pcre.split ~rex:(Pcre.regexp "[\r]?\n") icy_headers in
+        let icy_headers =
+          Re.Pcre.split ~rex:(Re.Pcre.regexp "[\r]?\n") icy_headers
+        in
         List.fold_left
           (fun ret header ->
             if header <> "" then (
               try
-                let res = Pcre.exec ~pat:"([^:]*):\\s*(.*)" header in
-                (Pcre.get_substring res 1, Pcre.get_substring res 2) :: ret
+                let res =
+                  Re.Pcre.exec ~rex:(Re.Pcre.regexp "([^:]*):\\s*(.*)") header
+                in
+                (Re.Pcre.get_substring res 1, Re.Pcre.get_substring res 2)
+                :: ret
               with Not_found -> ret)
             else ret)
           [] icy_headers
@@ -320,8 +329,8 @@ class http_input ~autostart ~self_sync ~poll_delay ~debug ~clock_safe ~on_error
   object
     inherit
       input
-        ~name:"input.http" ~autostart ~self_sync ~poll_delay ~debug ~clock_safe
-          ~max_buffer ~on_stop ~on_start ~on_disconnect ~on_connect ~on_error
+        ~name:"input.http" ~autostart ~self_sync ~poll_delay ~debug ~max_buffer
+          ~on_stop ~on_start ~on_disconnect ~on_connect ~on_error
           ~metadata_filter ?format ~opts ~new_track_on_metadata ~trim_url url
   end
 
@@ -356,12 +365,11 @@ let register_input is_http =
     else ("ffmpeg", "Create a stream using ffmpeg")
   in
   let on_error =
-    Liquidsoap_lang.Runtime.eval ~ignored:false ~ty:(Lang.univ_t ())
-      "fun (_) -> ()"
+    Lang.eval ~cache:false ~stdlib:`Disabled ~typecheck:false "fun (_) -> ()"
   in
   ignore
     (Lang.add_operator ~base:Modules.input name ~descr ~category:`Input
-       (Start_stop.active_source_proto ~clock_safe:false ~fallible_opt:`Nope
+       (Start_stop.active_source_proto ~fallible_opt:`Nope
        @ (if is_http then
             [
               ( "user_agent",
@@ -493,8 +501,8 @@ let register_input is_http =
                            | `Starting -> "starting"
                            | `Stopping -> "stopping"
                            | `Polling -> "polling"
-                           | `Connected url -> Printf.sprintf "connected %s" url))
-               );
+                           | `Connected (url, _) ->
+                               Printf.sprintf "connected %s" url)) );
                ( "buffer_length",
                  ([], fun_t [] float_t),
                  "Get the buffer's length in seconds.",
@@ -528,7 +536,6 @@ let register_input is_http =
            else Some (Lang.to_bool (self_sync ()))
          in
          let autostart = Lang.to_bool (List.assoc "start" p) in
-         let clock_safe = Lang.to_bool (List.assoc "clock_safe" p) in
          let on_error =
            let f = List.assoc "on_error" p in
            fun err -> ignore (Lang.apply f [("", Lang.error err)])
@@ -552,7 +559,7 @@ let register_input is_http =
                      (Lang.apply fn [("", Lang.metadata_list m)])
              | None ->
                  List.filter (fun (k, _) ->
-                     not (Pcre.pmatch ~pat:"^id3v2_priv" k))
+                     not (Re.Pcre.pmatch ~rex:(Re.Pcre.regexp "^id3v2_priv") k))
          in
          let deduplicate_metadata =
            Lang.to_bool (List.assoc "deduplicate_metadata" p)
@@ -570,11 +577,8 @@ let register_input is_http =
                  m))
          in
          let metadata_filter m =
-           let m = Hashtbl.fold (fun k v m -> (k, v) :: m) m [] in
-           let m = metadata_filter m in
-           let ret = Hashtbl.create (List.length m) in
-           List.iter (fun (k, v) -> Hashtbl.add ret k v) m;
-           ret
+           let m = metadata_filter (Frame.Metadata.to_list m) in
+           Frame.Metadata.from_list m
          in
          let new_track_on_metadata =
            Lang.to_bool (List.assoc "new_track_on_metadata" p)
@@ -595,19 +599,19 @@ let register_input is_http =
              ignore (Lang.apply (List.assoc "on_connect" p) [("", arg)])
            in
            (new http_input
-              ~metadata_filter ~debug ~autostart ~self_sync ~clock_safe
-              ~poll_delay ~on_connect ~on_disconnect ~user_agent
-              ~new_track_on_metadata ~max_buffer ?format ~opts ~timeout
-              ~on_error ~on_start ~on_stop ~trim_url url
+              ~metadata_filter ~debug ~autostart ~self_sync ~poll_delay
+              ~on_connect ~on_disconnect ~user_agent ~new_track_on_metadata
+              ~max_buffer ?format ~opts ~timeout ~on_error ~on_start ~on_stop
+              ~trim_url url
              :> input))
          else (
            let on_connect _ =
              ignore (Lang.apply (List.assoc "on_connect" p) [])
            in
            new input
-             ~metadata_filter ~autostart ~debug ~self_sync ~clock_safe
-             ~poll_delay ~on_error ~on_start ~on_stop ~on_connect ~on_disconnect
-             ~max_buffer ?format ~opts ~new_track_on_metadata ~trim_url url)))
+             ~metadata_filter ~autostart ~debug ~self_sync ~poll_delay ~on_error
+             ~on_start ~on_stop ~on_connect ~on_disconnect ~max_buffer ?format
+             ~opts ~new_track_on_metadata ~trim_url url)))
 
 let () =
   register_input true;
