@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,6 +25,14 @@
 exception Done
 exception Not_connected
 
+module SyncSource = Clock.MkSyncSource (struct
+  type t = unit
+
+  let to_string _ = "srt"
+end)
+
+let sync_source = SyncSource.make ()
+
 let mode_of_value v =
   match Lang.to_string v with
     | "listener" -> `Listener
@@ -119,7 +127,7 @@ let common_options ~mode =
       Some Lang.null,
       Some
         "Timeout, in seconds, after which initial connection operations are \
-         aborted if no data was received. Uses library's default if `nulll`. \
+         aborted if no data was received. Uses library's default if `null`. \
          Used only in `client` mode." );
     ("payload_size", Lang.int_t, Some (Lang.int 1316), Some "Payload size.");
     ("messageapi", Lang.bool_t, Some (Lang.bool true), Some "Use message api");
@@ -182,9 +190,18 @@ let parse_common_options p =
            ( List.assoc "bind_address" p,
              Printf.sprintf "Invalid address: %s" (Printexc.to_string exn) ))
   in
-  let passphrase =
-    Lang.to_valued_option Lang.to_string (List.assoc "passphrase" p)
-  in
+  let passphrase_v = List.assoc "passphrase" p in
+  let passphrase = Lang.to_valued_option Lang.to_string passphrase_v in
+  (match passphrase with
+    | Some s when String.length s < 10 ->
+        raise
+          (Error.Invalid_value
+             (passphrase_v, "Passphrase must be at least 10 characters long!"))
+    | Some s when String.length s > 79 ->
+        raise
+          (Error.Invalid_value
+             (passphrase_v, "Passphrase must be at most 79 characters long!"))
+    | _ -> ());
   let streamid =
     Lang.to_valued_option Lang.to_string (List.assoc "streamid" p)
   in
@@ -282,7 +299,12 @@ let conf_enforced_encryption =
 let log = Log.make ["srt"]
 
 let log_handler { Srt.Log.message } =
-  let message = Pcre.substitute ~pat:"[ \r\n]+$" ~subst:(fun _ -> "") message in
+  let message =
+    Re.Pcre.substitute
+      ~rex:(Re.Pcre.regexp "[ \r\n]+$")
+      ~subst:(fun _ -> "")
+      message
+  in
   log#f conf_level#get "%s" message
 
 (** Common polling task for all srt input/output.
@@ -341,7 +363,7 @@ module Poll = struct
   let add_socket ~mode socket fn =
     Srt.setsockflag socket Srt.sndsyn false;
     Srt.setsockflag socket Srt.rcvsyn false;
-    Hashtbl.add t.handlers socket (mode, fn);
+    Hashtbl.replace t.handlers socket (mode, fn);
     Srt.Poll.add_usock t.p socket ~flags:[(mode :> Srt.Poll.flag)];
     Duppy.Async.wake_up task
 
@@ -350,26 +372,28 @@ module Poll = struct
       Srt.Poll.remove_usock t.p socket
 end
 
-let () =
-  Lifecycle.on_start ~name:"srt initialization" (fun () ->
-      Srt.startup ();
-      if conf_log#get then (
-        let level =
-          match conf_verbosity#get with
-            | "critical" -> `Critical
-            | "error" -> `Error
-            | "warning" -> `Warning
-            | "notice" -> `Notice
-            | "debug" -> `Debug
-            | _ ->
-                log#severe "Invalid value for \"srt.log.verbosity\"!";
-                `Error
-        in
-        Srt.Log.setloglevel level;
-        Srt.Log.set_handler log_handler));
-  Lifecycle.on_final_cleanup ~name:"set cleanup" (fun () ->
-      Srt.Poll.release Poll.t.Poll.p;
-      Srt.cleanup ())
+let init =
+  lazy
+    (Lifecycle.on_start ~name:"srt initialization" (fun () ->
+         Srt.startup ();
+         if conf_log#get then (
+           let level =
+             match conf_verbosity#get with
+               | "critical" -> `Critical
+               | "error" -> `Error
+               | "warning" -> `Warning
+               | "notice" -> `Notice
+               | "debug" -> `Debug
+               | _ ->
+                   log#severe "Invalid value for \"srt.log.verbosity\"!";
+                   `Error
+           in
+           Srt.Log.setloglevel level;
+           Srt.Log.set_handler log_handler));
+
+     Lifecycle.on_final_cleanup ~name:"set cleanup" (fun () ->
+         Srt.Poll.release Poll.t.Poll.p;
+         Srt.cleanup ()))
 
 let string_of_address = function
   | Unix.ADDR_UNIX _ -> assert false
@@ -395,7 +419,8 @@ let id =
   let counter = Atomic.make 0 in
   fun () -> Atomic.fetch_and_add counter 1
 
-class virtual base =
+class virtual base () =
+  let () = Lazy.force init in
   object
     val should_stop = Atomic.make false
     val id = id ()
@@ -428,10 +453,11 @@ class virtual output_networking_agent =
         : Srt.socket -> exn -> Printexc.raw_backtrace -> unit
   end
 
-module ToDisconnect = Liquidsoap_lang.Active_value.Make (struct
+module ToDisconnect = Weak.Make (struct
   type t = < disconnect : unit ; srt_id : int >
 
-  let id t = t#srt_id
+  let equal t t' = t#srt_id = t'#srt_id
+  let hash t = t#srt_id
 end)
 
 let to_disconnect = ToDisconnect.create 10
@@ -631,29 +657,26 @@ class virtual listener ~enforced_encryption ~pbkeylen ~passphrase ~max_clients
         (fun () ->
           List.iter (fun (_, s) -> close_socket s) client_sockets;
           client_sockets <- [];
-          if should_stop then (
-            ignore
-              (Option.map
-                 (fun s ->
-                   Poll.remove_socket s;
-                   close_socket s)
-                 (Atomic.get listening_socket));
-            Atomic.set listening_socket None);
+          (match (should_stop, Atomic.get listening_socket) with
+            | true, Some s ->
+                Poll.remove_socket s;
+                close_socket s;
+                Atomic.set listening_socket None
+            | _ -> ());
           !on_disconnect ())
         ()
   end
 
-class virtual input_base ~max ~self_sync ~clock_safe ~on_connect ~on_disconnect
+class virtual input_base ~max ~self_sync ~on_connect ~on_disconnect
   ~payload_size ~dump ~on_start ~on_stop ~autostart format =
   let max_length = Some (Frame.main_of_seconds max) in
   object (self)
     inherit input_networking_agent
-    inherit base
+    inherit base ()
 
     inherit
       Start_stop.active_source
-        ~name:"input.srt" ~clock_safe ~on_start ~on_stop ~autostart
-          ~fallible:true () as super
+        ~name:"input.srt" ~on_start ~on_stop ~autostart ~fallible:true () as super
 
     val mutable decoder_data = None
     val mutable dump_chan = None
@@ -669,6 +692,9 @@ class virtual input_base ~max ~self_sync ~clock_safe ~on_connect ~on_disconnect
       let on_disconnect_cur = !on_disconnect in
       on_disconnect :=
         fun () ->
+          (match decoder_data with
+            | Some (d, _) -> d.Decoder.close ()
+            | None -> ());
           decoder_data <- None;
           (match dump_chan with
             | Some chan ->
@@ -677,19 +703,21 @@ class virtual input_base ~max ~self_sync ~clock_safe ~on_connect ~on_disconnect
             | None -> ());
           on_disconnect_cur ()
 
-    method! wake_up act =
-      super#wake_up act;
-      Generator.set_max_length self#buffer max_length
+    initializer
+      self#on_wake_up (fun () ->
+          Generator.set_max_length self#buffer max_length)
 
     method seek_source = (self :> Source.source)
     method remaining = -1
     method abort_track = Generator.add_track_mark self#buffer
 
-    method! is_ready ?frame () =
-      super#is_ready ?frame () && (not self#should_stop) && self#is_connected
+    method private can_generate_frame =
+      super#started && (not self#should_stop) && self#is_connected
 
     method self_sync =
-      if self_sync then (`Dynamic, self#is_connected) else (`Static, false)
+      if self_sync then
+        (`Dynamic, if self#is_connected then Some sync_source else None)
+      else (`Static, None)
 
     method private create_decoder socket =
       let create_decoder =
@@ -719,8 +747,8 @@ class virtual input_base ~max ~self_sync ~clock_safe ~on_connect ~on_disconnect
       in
       create_decoder { Decoder.read; tell = None; length = None; lseek = None }
 
-    method private get_frame frame =
-      let pos = Frame.position frame in
+    method private generate_frame =
+      let size = Lazy.force Frame.size in
       try
         let _, socket = self#get_socket in
         let decoder, buffer =
@@ -731,20 +759,21 @@ class virtual input_base ~max ~self_sync ~clock_safe ~on_connect ~on_disconnect
                 in
                 let decoder = self#create_decoder socket in
                 decoder_data <- Some (decoder, buffer);
+                Generator.add_track_mark self#buffer;
                 (decoder, buffer)
             | Some d -> d
         in
-        while Generator.length self#buffer < Lazy.force Frame.size do
+        while Generator.length self#buffer < size do
           decoder.Decoder.decode buffer
         done;
-        Generator.fill self#buffer frame
+        Generator.slice self#buffer size
       with exn ->
         let bt = Printexc.get_backtrace () in
         Utils.log_exception ~log:self#log ~bt
           (Printf.sprintf "Feeding failed: %s" (Printexc.to_string exn));
         self#disconnect;
         if not self#should_stop then self#connect;
-        Frame.add_break frame pos
+        Frame.append (Generator.slice self#buffer size) self#end_of_track
 
     method private start =
       self#set_should_stop false;
@@ -756,14 +785,14 @@ class virtual input_base ~max ~self_sync ~clock_safe ~on_connect ~on_disconnect
   end
 
 class input_listener ~enforced_encryption ~pbkeylen ~passphrase ~listen_callback
-  ~bind_address ~max ~payload_size ~self_sync ~clock_safe ~on_connect
-  ~on_disconnect ~read_timeout ~write_timeout ~messageapi ~dump ~on_start
-  ~on_stop ~autostart format =
+  ~bind_address ~max ~payload_size ~self_sync ~on_connect ~on_disconnect
+  ~read_timeout ~write_timeout ~messageapi ~dump ~on_start ~on_stop ~autostart
+  format =
   object (self)
     inherit
       input_base
-        ~max ~payload_size ~self_sync ~clock_safe ~on_connect ~on_disconnect
-          ~dump ~on_start ~on_stop ~autostart format
+        ~max ~payload_size ~self_sync ~on_connect ~on_disconnect ~dump ~on_start
+          ~on_stop ~autostart format
 
     inherit
       listener
@@ -779,14 +808,14 @@ class input_listener ~enforced_encryption ~pbkeylen ~passphrase ~listen_callback
   end
 
 class input_caller ~enforced_encryption ~pbkeylen ~passphrase ~streamid
-  ~polling_delay ~hostname ~port ~max ~payload_size ~self_sync ~clock_safe
-  ~on_connect ~on_disconnect ~read_timeout ~write_timeout ~connection_timeout
-  ~messageapi ~dump ~on_start ~on_stop ~autostart format =
+  ~polling_delay ~hostname ~port ~max ~payload_size ~self_sync ~on_connect
+  ~on_disconnect ~read_timeout ~write_timeout ~connection_timeout ~messageapi
+  ~dump ~on_start ~on_stop ~autostart format =
   object (self)
     inherit
       input_base
-        ~max ~payload_size ~self_sync ~clock_safe ~on_connect ~on_disconnect
-          ~dump ~on_start ~on_stop ~autostart format
+        ~max ~payload_size ~self_sync ~on_connect ~on_disconnect ~dump ~on_start
+          ~on_stop ~autostart format
 
     inherit
       caller
@@ -804,7 +833,7 @@ let _ =
     ~meth:(meth () @ Start_stop.meth ())
     ~descr:"Receive a SRT stream from a distant agent."
     (common_options ~mode:`Listener
-    @ Start_stop.active_source_proto ~clock_safe:true ~fallible_opt:`Nope
+    @ Start_stop.active_source_proto ~fallible_opt:`Nope
     @ [
         ( "max",
           Lang.float_t,
@@ -858,7 +887,6 @@ let _ =
           | s -> Some s
       in
       let max = Lang.to_float (List.assoc "max" p) in
-      let clock_safe = Lang.to_bool (List.assoc "clock_safe" p) in
       let self_sync = Lang.to_bool (List.assoc "self_sync" p) in
       let on_start =
         let f = List.assoc "on_start" p in
@@ -875,17 +903,17 @@ let _ =
             (new input_listener
                ~enforced_encryption ~pbkeylen ~passphrase ~listen_callback
                ~bind_address ~read_timeout ~write_timeout ~payload_size
-               ~self_sync ~clock_safe ~on_connect ~on_disconnect ~messageapi
-               ~max ~dump ~on_start ~on_stop ~autostart format
+               ~self_sync ~on_connect ~on_disconnect ~messageapi ~max ~dump
+               ~on_start ~on_stop ~autostart format
               :> < Start_stop.active_source
                  ; get_sockets : (Unix.sockaddr * Srt.socket) list >)
         | `Caller ->
             (new input_caller
                ~enforced_encryption ~pbkeylen ~passphrase ~streamid
                ~polling_delay ~hostname ~port ~payload_size ~self_sync
-               ~clock_safe ~on_connect ~read_timeout ~write_timeout
-               ~connection_timeout ~on_disconnect ~messageapi ~max ~dump
-               ~on_start ~on_stop ~autostart format
+               ~on_connect ~read_timeout ~write_timeout ~connection_timeout
+               ~on_disconnect ~messageapi ~max ~dump ~on_start ~on_stop
+               ~autostart format
               :> < Start_stop.active_source
                  ; get_sockets : (Unix.sockaddr * Srt.socket) list >))
 
@@ -896,10 +924,10 @@ class virtual output_base ~payload_size ~messageapi ~on_start ~on_stop
   let tmp = Bytes.create payload_size in
   object (self)
     inherit output_networking_agent
-    inherit base
+    inherit base ()
 
     inherit
-      Output.encoded
+      [Strings.t] Output.encoded
         ~output_kind:"srt" ~on_start ~on_stop ~infallible ~register_telnet
           ~export_cover_metadata:false ~autostart ~name:"output.srt" source
 
@@ -950,9 +978,7 @@ class virtual output_base ~payload_size ~messageapi ~on_start ~on_stop
           match encoder with
             | Some enc -> enc
             | None ->
-                let enc =
-                  encoder_factory self#id Export_metadata.empty_metadata
-                in
+                let enc = encoder_factory self#id Frame.Metadata.Export.empty in
                 encoder <- Some enc;
                 enc)
         ()
@@ -969,8 +995,8 @@ class virtual output_base ~payload_size ~messageapi ~on_start ~on_stop
       self#mutexify (fun () -> Atomic.set should_stop true) ();
       self#disconnect
 
-    method private encode frame ofs len =
-      if self#is_connected then self#get_encoder.Encoder.encode frame ofs len
+    method private encode frame =
+      if self#is_connected then self#get_encoder.Encoder.encode frame
       else Strings.empty
 
     method private insert_metadata m =
@@ -1099,7 +1125,7 @@ let _ =
       let format_val = Lang.assoc "" 1 p in
       let format = Lang.to_format format_val in
       let encoder_factory =
-        try (Encoder.get_factory format) ~pos:format_val.Value.pos
+        try (Encoder.get_factory format) ~pos:(Value.pos format_val)
         with Not_found ->
           raise
             (Error.Invalid_value
