@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -32,14 +32,13 @@ class compress ~attack ~release ~threshold ~ratio ~knee ~track_sensitive
   ~pre_gain ~make_up_gain ~lookahead ~window ~wet ~field (source : source) =
   let lookahead () = Frame.audio_of_seconds (lookahead ()) in
   object (self)
-    inherit operator ~name:"compress" [source] as super
+    inherit operator ~name:"compress" [source]
     val mutable effect = None
-    method! private wake_up a = super#wake_up a
-    method stype = source#stype
+    method fallible = source#fallible
     method remaining = source#remaining
     method seek_source = source#seek_source
     method self_sync = source#self_sync
-    method private _is_ready = source#is_ready
+    method private can_generate_frame = source#is_ready
     method abort_track = source#abort_track
 
     (* Current gain in dB. *)
@@ -65,13 +64,10 @@ class compress ~attack ~release ~threshold ~ratio ~knee ~track_sensitive
       gain <- 0.;
       ms <- 0.
 
-    method private get_frame buf =
-      let ofs = AFrame.position buf in
-      source#get buf;
-      let pos = AFrame.position buf in
-      let partial = AFrame.is_partial buf in
-      let buf = Content.Audio.get_data (Frame.get buf field) in
-      let chans = self#audio_channels in
+    method private compress frame =
+      let pos = AFrame.position frame in
+      let buf = Content.Audio.get_data (Frame.get frame field) in
+      let chans = Array.length buf in
       let samplerate = float (Lazy.force Frame.audio_rate) in
       let threshold = threshold () in
       let knee = knee () in
@@ -88,7 +84,7 @@ class compress ~attack ~release ~threshold ~ratio ~knee ~track_sensitive
       let window_coef = 1. -. exp (-1. /. (window *. samplerate)) in
       let wet = wet () in
       self#prepare lookahead;
-      for i = ofs to pos - 1 do
+      for i = 0 to pos - 1 do
         (* Apply pre_gain. *)
         if pre_gain <> 0. then
           for c = 0 to chans - 1 do
@@ -163,7 +159,15 @@ class compress ~attack ~release ~threshold ~ratio ~knee ~track_sensitive
           buf.(c).(i) <- buf.(c).(i) *. (1. -. wet +. (wet *. gain))
         done
       done;
-      if partial && track_sensitive then self#reset
+      Frame.set_data frame field Content.Audio.lift_data buf
+
+    method private generate_frame =
+      match self#split_frame (source#get_mutable_frame field) with
+        | frame, None -> self#compress frame
+        | frame, Some new_track ->
+            let frame = self#compress frame in
+            if track_sensitive then self#reset;
+            Frame.append frame (self#compress new_track)
   end
 
 let audio_compress =
@@ -243,7 +247,7 @@ let audio_compress =
       let ratio =
         let pos = Lang.pos p in
         match List.assoc "ratio" p with
-          | Liquidsoap_lang.Value.{ value = Ground (Ground.Float f) } ->
+          | Liquidsoap_lang.Value.Float { value = f } ->
               if f < 1. then
                 Runtime_error.raise ~pos ~message:"Ratio must be at least 1!"
                   "eval";
