@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,6 +22,8 @@
 
 (** Output to an icecast server. *)
 
+module Http = Liq_http
+
 let error_translator = function
   | Cry.Error _ as e -> Some (Cry.string_of_error e)
   | _ -> None
@@ -78,6 +80,8 @@ module Icecast = struct
             samplerate = Some (Lazy.force m.Fdkaac_format.samplerate);
             channels = Some m.Fdkaac_format.channels;
           }
+      | Encoder.NDI _ ->
+          { quality = None; bitrate = None; samplerate = None; channels = None }
       | Encoder.External m ->
           {
             quality = None;
@@ -85,13 +89,6 @@ module Icecast = struct
             samplerate = Some (Lazy.force m.External_encoder_format.samplerate);
             channels = Some m.External_encoder_format.channels;
           }
-      | Encoder.GStreamer gst ->
-          {
-            quality = None;
-            bitrate = None;
-            samplerate = None;
-            channels = Some (Gstreamer_format.audio_channels gst);
-          }
       | Encoder.Flac m ->
           {
             quality = Some (string_of_int m.Flac_format.compression);
@@ -193,8 +190,8 @@ open M
 let user_agent =
   Lang.product (Lang.string "User-Agent") (Lang.string Http.user_agent)
 
-let default_icy_song () =
-  Liquidsoap_lang.Runtime.eval ~ignored:false ~ty:(Lang.univ_t ())
+let default_icy_song =
+  Lang.eval ~cache:false ~typecheck:false ~stdlib:`Disabled
     {|fun (m) -> begin
   title = m["title"]
   artist = m["artist"]
@@ -293,7 +290,7 @@ let proto frame_t =
         Lang.fun_t
           [(false, "", Lang.metadata_t)]
           (Lang.nullable_t Lang.string_t),
-        Some (default_icy_song ()),
+        Some default_icy_song,
         Some
           "Function used to generate the default icy \"song\" metadata. \
            Metadata is not added when returning `null`. Default: `$(artist) - \
@@ -480,7 +477,7 @@ class output p =
   let connection = Cry.create ~timeout ~transport ?connection_timeout () in
   object (self)
     inherit
-      Output.encoded
+      [Strings.t] Output.encoded
         ~output_kind:"output.icecast" ~infallible ~register_telnet ~autostart
           ~export_cover_metadata:false ~on_start ~on_stop ~name source
 
@@ -503,12 +500,12 @@ class output p =
 
     val mutable encoder = None
 
-    method encode frame ofs len =
+    method encode frame =
       (* We assume here that there always is
        * an encoder available when the source
        * is connected. *)
       match (Cry.get_status connection, encoder) with
-        | Cry.Connected _, Some enc -> enc.Encoder.encode frame ofs len
+        | Cry.Connected _, Some enc -> enc.Encoder.encode frame
         | _ -> Strings.empty
 
     method insert_metadata m =
@@ -516,15 +513,15 @@ class output p =
       if send_icy_metadata then (
         let f = Charset.convert ~target:out_enc in
         let icy_meta = Hashtbl.create 10 in
-        let m = Export_metadata.to_metadata m in
-        Hashtbl.iter
+        let m = Frame.Metadata.Export.to_metadata m in
+        Frame.Metadata.iter
           (fun lbl v ->
             if List.mem lbl icy_metadata then Hashtbl.replace icy_meta lbl (f v))
           m;
         if not (Hashtbl.mem icy_meta "song") then (
           match icy_song (Lang.metadata m) with
             | None -> ()
-            | Some v -> Hashtbl.add icy_meta "song" (f v));
+            | Some v -> Hashtbl.replace icy_meta "song" (f v));
         (* Do nothing if shout connection isn't available *)
         match Cry.get_status connection with
           | Cry.Connected _ -> (
@@ -578,7 +575,7 @@ class output p =
 
     method icecast_start =
       assert (encoder = None);
-      let enc = data.factory self#id Export_metadata.empty_metadata in
+      let enc = data.factory self#id Frame.Metadata.Export.empty in
       encoder <- Some enc;
       assert (Cry.get_status connection = Cry.Disconnected);
       begin
@@ -595,7 +592,7 @@ class output p =
         host;
       let audio_info = Hashtbl.create 10 in
       let f x y z =
-        match x with Some q -> Hashtbl.add audio_info y (z q) | None -> ()
+        match x with Some q -> Hashtbl.replace audio_info y (z q) | None -> ()
       in
       let info = data.info enc in
       f info.bitrate "bitrate" string_of_int;
