@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -21,19 +21,17 @@
  *****************************************************************************)
 
 let log = Log.make ["playlist"; "basic"]
-let split_lines buf = Pcre.split ~pat:"[\r\n]+" buf
+let split_lines buf = Re.Pcre.split ~rex:(Re.Pcre.regexp "[\r\n]+") buf
 
 let parse_meta =
+  let processor =
+    MenhirLib.Convert.Simplified.traditional2revised
+      Liquidsoap_lang.Parser.annotate_metadata_entry
+  in
   let rec f cur s =
     try
       let lexbuf = Sedlexing.Utf8.from_string s in
-      let processor =
-        MenhirLib.Convert.Simplified.traditional2revised
-          Liquidsoap_lang.Parser.annotate_metadata_entry
-      in
-      let tokenizer =
-        Liquidsoap_lang.Preprocessor.mk_tokenizer ~pwd:"" lexbuf
-      in
+      let tokenizer = Liquidsoap_lang.Preprocessor.mk_tokenizer lexbuf in
       let metadata = processor tokenizer in
       let b = Buffer.create 10 in
       let rec g () =
@@ -50,34 +48,35 @@ let parse_meta =
 
 let parse_extinf s =
   try
-    let rex = Pcre.regexp "#EXTINF:(\\d*)\\s*(.*)" in
-    let sub = Pcre.exec ~rex s in
+    let rex = Re.Pcre.regexp "#EXTINF:(\\d*)\\s*(.*)" in
+    let sub = Re.Pcre.exec ~rex s in
     let meta, song =
-      match Pcre.get_substring sub 2 with
+      match Re.Pcre.get_substring sub 2 with
         | "" -> ([], "")
         | s when s.[0] = ',' -> ([], String.sub s 1 (String.length s - 1))
         | s -> parse_meta s
     in
     let meta =
-      match Pcre.get_substring sub 1 with
+      match Re.Pcre.get_substring sub 1 with
         | "" -> meta
         | duration -> ("extinf_duration", duration) :: meta
     in
-    let lines = Pcre.split ~pat:"\\s*-\\s*" song in
+    let lines = Re.Pcre.split ~rex:(Re.Pcre.regexp "\\s*-\\s*") song in
     meta
     @
     match lines with
-      | [] -> []
+      | [] | [""; ""] -> []
       | [""; song] -> [("song", String.trim song)]
       | [artist; title] ->
           [("artist", String.trim artist); ("title", String.trim title)]
+      | _ when song = "" -> []
       | _ -> [("song", String.trim song)]
   with Not_found -> []
 
 (* This parser cannot detect the format !! *)
 let parse_mpegurl ?pwd string =
   let lines = List.filter (fun x -> x <> "") (split_lines string) in
-  let is_info line = Pcre.pmatch ~pat:"^#EXTINF" line in
+  let is_info line = Re.Pcre.pmatch ~rex:(Re.Pcre.regexp "^#EXTINF") line in
   let skip_line line = line.[0] == '#' in
   let rec get_urls cur lines =
     match lines with
@@ -92,10 +91,16 @@ let parse_mpegurl ?pwd string =
   get_urls [] lines
 
 let parse_scpls ?pwd string =
-  let string = Pcre.replace ~pat:"#[^\\r\\n]*[\\n\\r]+" string in
+  let string =
+    Re.Pcre.substitute
+      ~rex:(Re.Pcre.regexp "#[^\\r\\n]*[\\n\\r]+")
+      ~subst:(fun _ -> "")
+      string
+  in
   (* Format check, raise Not_found if invalid *)
   ignore
-    (Pcre.exec ~pat:"^[\\r\\n\\s]*\\[playlist\\]"
+    (Re.Pcre.exec
+       ~rex:(Re.Pcre.regexp "^[\\r\\n\\s]*\\[playlist\\]")
        (String.lowercase_ascii string));
   let lines = split_lines string in
   let urls =
@@ -103,10 +108,10 @@ let parse_scpls ?pwd string =
       (fun s ->
         try
           let rex =
-            Pcre.regexp ~flags:[`CASELESS] "file\\d*\\s*=\\s*(.*)\\s*"
+            Re.Pcre.regexp ~flags:[`CASELESS] "file\\d*\\s*=\\s*(.*)\\s*"
           in
-          let sub = Pcre.exec ~rex s in
-          Pcre.get_substring sub 1
+          let sub = Re.Pcre.exec ~rex s in
+          Re.Pcre.get_substring sub 1
         with Not_found -> "")
       lines
   in
@@ -209,7 +214,7 @@ let parse_tracks index lines =
                   else track)
             in
             begin
-              try parse_index x (Hashtbl.add track.indexes)
+              try parse_index x (Hashtbl.replace track.indexes)
               with Not_found -> ()
             end;
             parse tracks track rem)
@@ -228,7 +233,10 @@ let parse_cue ?pwd string =
   let strings = split_lines string in
   let strings =
     List.map
-      (fun string -> Pcre.replace ~rex:(Pcre.regexp "^\\s+") string)
+      (fun string ->
+        Re.Pcre.substitute ~rex:(Re.Pcre.regexp "^\\s+")
+          ~subst:(fun _ -> "")
+          string)
       strings
   in
   let strings = List.filter (fun s -> s <> "") strings in
