@@ -1,7 +1,7 @@
 (*****************************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -71,7 +71,7 @@ let exit () =
 let generic_queues =
   Dtools.Conf.int
     ~p:(conf_scheduler#plug "generic_queues")
-    ~d:2 "Generic queues"
+    ~d:5 "Generic queues"
     ~comments:
       [
         "Number of event queues accepting any kind of task.";
@@ -110,18 +110,6 @@ let scheduler_log =
     ~p:(conf_scheduler#plug "log")
     ~d:false "Log scheduler messages"
 
-let mutexify lock f x =
-  Mutex.lock lock;
-  try
-    let v = f x in
-    Mutex.unlock lock;
-    v
-  with exn ->
-    let bt = Printexc.get_raw_backtrace () in
-    Mutex.unlock lock;
-    Printexc.raise_with_backtrace exn bt
-  [@@inline always]
-
 let seems_locked =
   if Sys.win32 then fun _ -> true
   else fun m ->
@@ -149,7 +137,7 @@ let queues = ref Set.empty
 let join_all ~set () =
   let rec f () =
     try
-      mutexify lock
+      Mutex_utils.mutexify lock
         (fun () ->
           let name, c = Set.choose !set in
           log#info "Waiting for thread %s to shutdown" name;
@@ -178,13 +166,14 @@ exception Exit
 let create ~queue f x s =
   let c = Condition.create () in
   let set = if queue then queues else all in
-  mutexify lock
+  Mutex_utils.mutexify lock
     (fun () ->
       let id =
         let process x =
+          Utils.Thread.set_current_thread_name s;
           try
             f x;
-            mutexify lock
+            Mutex_utils.mutexify lock
               (fun () ->
                 set := Set.remove (s, c) !set;
                 log#info "Thread %S terminated (%d remaining)." s
@@ -216,9 +205,9 @@ let create ~queue f x s =
                       (Printexc.to_string e);
                     Printexc.raise_with_backtrace e raw_bt
             with e ->
-              let l = Pcre.split ~pat:"\n" bt in
+              let l = String.split_on_char '\n' bt in
               List.iter (log#info "%s") l;
-              mutexify lock
+              Mutex_utils.mutexify lock
                 (fun () ->
                   set := Set.remove (s, c) !set;
                   if
@@ -306,7 +295,7 @@ let start () =
 
 (** Waits for [f()] to become true on condition [c]. *)
 let wait c m f =
-  mutexify m
+  Mutex_utils.mutexify m
     (fun () ->
       while not (f ()) do
         Condition.wait c m
@@ -385,18 +374,6 @@ let cleanup () =
   join_all ();
   log#important "Main threads terminated."
 
-(** Thread-safe lazy cell. *)
-let lazy_cell f =
-  let lock = Mutex.create () in
-  let c = ref None in
-  mutexify lock (fun () ->
-      match !c with
-        | Some v -> v
-        | None ->
-            let v = f () in
-            c := Some v;
-            v)
-
 let write_all ?timeout fd b =
   let rec f ofs len =
     (match timeout with
