@@ -1,7 +1,7 @@
-(*****************************************************************************
+(**********************************************************************
 
-  Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2023 Savonet team
+  Liquidsoap, a programmable stream generator.
+  Copyright 2003-2024 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -32,6 +32,13 @@ let log_exception ~(log : Log.t) ~bt msg =
   log#severe "%s" msg;
   if log#active 4 (* info *) then log#info "%s" bt
 
+module Thread = struct
+  external set_current_thread_name : string -> unit
+    = "liquidsoap_set_current_thread_name"
+
+  include Thread
+end
+
 (* Force locale *)
 external force_locale : string -> unit = "liquidsoap_set_locale"
 
@@ -50,18 +57,6 @@ let rec prefix p l =
     | _, [] -> false
     | hp :: tp, hl :: tl -> hp = hl && prefix tp tl
 
-let hashtbl_of_list l =
-  let h = Hashtbl.create (List.length l) in
-  List.iter (fun (k, v) -> Hashtbl.add h k v) l;
-  h
-
-let list_of_metadata m =
-  let f x y l = (x, y) :: l in
-  List.sort (fun (k, _) (k', _) -> Stdlib.compare k k') (Hashtbl.fold f m [])
-
-let hashtbl_get : ('a, 'b) Hashtbl.t -> 'a -> 'b option =
- fun h k -> try Some (Hashtbl.find h k) with Not_found -> None
-
 (** Remove the first element satisfying a predicate, raising Not_found
   * if none is found. *)
 let remove_one f l =
@@ -296,10 +291,10 @@ let strftime ?time str : string =
     ]
   in
   let subst sub =
-    let key = Pcre.get_substring sub 1 in
+    let key = Re.Pcre.get_substring sub 1 in
     try List.assoc key assoc with _ -> "%" ^ key
   in
-  Pcre.substitute_substrings ~pat:"%(.)" ~subst str
+  Re.replace (Re.Pcre.regexp "%(.)") ~f:subst str
 
 (** Check if a directory exists. *)
 let is_dir d =
@@ -332,9 +327,9 @@ let get_tempdir () =
 (** Get a file/uri extension. *)
 let get_ext s =
   try
-    let rex = Pcre.regexp "\\.([a-zA-Z0-9]+)[^.]*$" in
-    let ret = Pcre.exec ~rex s in
-    String.lowercase_ascii (Pcre.get_substring ret 1)
+    let rex = Re.Pcre.regexp "\\.([a-zA-Z0-9]+)[^.]*$" in
+    let ret = Re.Pcre.exec ~rex s in
+    String.lowercase_ascii (Re.Pcre.get_substring ret 1)
   with _ -> raise Not_found
 
 let get_ext_opt s = try Some (get_ext s) with Not_found -> None
@@ -359,17 +354,34 @@ let uptime =
 (** Generate a string which can be used as a parameter name. *)
 let normalize_parameter_string s =
   let s =
-    Pcre.substitute ~pat:"( *\\([^\\)]*\\)| *\\[[^\\]]*\\])"
+    Re.Pcre.substitute
+      ~rex:(Re.Pcre.regexp "( *\\([^\\)]*\\)| *\\[[^\\]]*\\])")
       ~subst:(fun _ -> "")
       s
   in
-  let s = Pcre.substitute ~pat:"(\\.+|\\++)" ~subst:(fun _ -> "") s in
-  let s = Pcre.substitute ~pat:" +$" ~subst:(fun _ -> "") s in
-  let s = Pcre.substitute ~pat:"( +|/+|-+)" ~subst:(fun _ -> "_") s in
-  let s = Pcre.substitute ~pat:"\"" ~subst:(fun _ -> "") s in
+  let s =
+    Re.Pcre.substitute
+      ~rex:(Re.Pcre.regexp "(\\.+|\\++)")
+      ~subst:(fun _ -> "")
+      s
+  in
+  let s =
+    Re.Pcre.substitute ~rex:(Re.Pcre.regexp " +$") ~subst:(fun _ -> "") s
+  in
+  let s =
+    Re.Pcre.substitute
+      ~rex:(Re.Pcre.regexp "( +|/+|-+)")
+      ~subst:(fun _ -> "_")
+      s
+  in
+  let s =
+    Re.Pcre.substitute ~rex:(Re.Pcre.regexp "\"") ~subst:(fun _ -> "") s
+  in
   let s = String.lowercase_ascii s in
   (* Identifiers cannot begin with a digit. *)
-  let s = if Pcre.pmatch ~pat:"^[0-9]" s then "_" ^ s else s in
+  let s =
+    if Re.Pcre.pmatch ~rex:(Re.Pcre.regexp "^[0-9]") s then "_" ^ s else s
+  in
   s
 
 (** A function to reopen a file descriptor
@@ -437,46 +449,6 @@ let string_of_size n =
     Printf.sprintf "%.02f MiB" (float_of_int n /. float_of_int (1 lsl 20))
   else Printf.sprintf "%.02f GiB" (float_of_int n /. float_of_int (1 lsl 30))
 
-let self_sync_type sources =
-  lazy
-    (fst
-       (List.fold_left
-          (fun cur s ->
-            match (cur, s#self_sync) with
-              | (`Static, None), (`Static, v) -> (`Static, Some v)
-              | (`Static, Some v), (`Static, v') when v = v' -> (`Static, Some v)
-              | _ -> (`Dynamic, None))
-          (`Static, None) sources))
-
-let string_of_pcre_error =
-  Pcre.(
-    function
-    | Partial -> "String only matched the pattern partially"
-    | BadPartial ->
-        "Pattern contains items that cannot be used together with partial \
-         matching."
-    | BadPattern (msg, pos) ->
-        Printf.sprintf "Malformed regular expression. Error: %s, position: %i"
-          msg pos
-    | BadUTF8 -> "UTF8 string being matched is invalid"
-    | BadUTF8Offset -> "A UTF8 string being matched with offset is invalid."
-    | MatchLimit ->
-        "Maximum allowed number of match attempts with backtracking or \
-         recursion is reached during matching."
-    | RecursionLimit -> "Maximum allowed number of recursion reached"
-    | InternalError msg -> Printf.sprintf "Internal error: %s" msg
-    (* This is a hack to be extensible here and enable warning 11 *)
-    | exn ->
-        if exn == WorkspaceSize then "Provided workspace array is too small"
-        else "Unknown error")
-
-let () =
-  Printexc.register_printer
-    Pcre.(
-      function
-      | Error err -> Some (Printf.sprintf "Pcre(%s)" (string_of_pcre_error err))
-      | _ -> None)
-
 let var_script = ref "default"
 
 let substs =
@@ -524,82 +496,7 @@ let frame_id_of_string = function
   | "tracknumber" -> Some `TRCK
   | "year" -> Some `TYER
   | "url" -> Some `WXXX
-  | "AENC" -> Some `AENC
-  | "APIC" -> Some `APIC
-  | "COMM" -> Some `COMM
-  | "COMR" -> Some `COMR
-  | "ENCR" -> Some `ENCR
-  | "EQUA" -> Some `EQUA
-  | "ETCO" -> Some `ETCO
-  | "GEOB" -> Some `GEOB
-  | "GRID" -> Some `GRID
-  | "IPLS" -> Some `IPLS
-  | "LINK" -> Some `LINK
-  | "MCDI" -> Some `MCDI
-  | "MLLT" -> Some `MLLT
-  | "OWNE" -> Some `OWNE
-  | "PRIV" -> Some `PRIV
-  | "PCNT" -> Some `PCNT
-  | "POPM" -> Some `POPM
-  | "POSS" -> Some `POSS
-  | "RBUF" -> Some `RBUF
-  | "RVAD" -> Some `RVAD
-  | "RVRB" -> Some `RVRB
-  | "SYLT" -> Some `SYLT
-  | "SYTC" -> Some `SYTC
-  | "TALB" -> Some `TALB
-  | "TBPM" -> Some `TBPM
-  | "TCOM" -> Some `TCOM
-  | "TCON" -> Some `TCON
-  | "TCOP" -> Some `TCOP
-  | "TDAT" -> Some `TDAT
-  | "TDLY" -> Some `TDLY
-  | "TENC" -> Some `TENC
-  | "TEXT" -> Some `TEXT
-  | "TFLT" -> Some `TFLT
-  | "TIME" -> Some `TIME
-  | "TIT1" -> Some `TIT1
-  | "TIT2" -> Some `TIT2
-  | "TIT3" -> Some `TIT3
-  | "TKEY" -> Some `TKEY
-  | "TLAN" -> Some `TLAN
-  | "TLEN" -> Some `TLEN
-  | "TMED" -> Some `TMED
-  | "TOAL" -> Some `TOAL
-  | "TOFN" -> Some `TOFN
-  | "TOLY" -> Some `TOLY
-  | "TOPE" -> Some `TOPE
-  | "TORY" -> Some `TORY
-  | "TOWN" -> Some `TOWN
-  | "TPE1" -> Some `TPE1
-  | "TPE2" -> Some `TPE2
-  | "TPE3" -> Some `TPE3
-  | "TPE4" -> Some `TPE4
-  | "TPOS" -> Some `TPOS
-  | "TPUB" -> Some `TPUB
-  | "TRCK" -> Some `TRCK
-  | "TRDA" -> Some `TRDA
-  | "TRSN" -> Some `TRSN
-  | "TRSO" -> Some `TRSO
-  | "TSIZ" -> Some `TSIZ
-  | "TSRC" -> Some `TSRC
-  | "TSSE" -> Some `TSSE
-  | "TYER" -> Some `TYER
-  | "TXXX" -> Some `TXXX
-  | "UFID" -> Some `UFID
-  | "USER" -> Some `USER
-  | "USLT" -> Some `USLT
-  | "WCOM" -> Some `WCOM
-  | "WCOP" -> Some `WCOP
-  | "WOAF" -> Some `WOAF
-  | "WOAR" -> Some `WOAR
-  | "WOAS" -> Some `WOAS
-  (* codespell 💩 *)
-  | x when x = "W" ^ "ORS" -> Some `WORS
-  | "WPAY" -> Some `WPAY
-  | "WPUB" -> Some `WPUB
-  | "WXXX" -> Some `WXXX
-  | _ -> None
+  | v -> Metadata.ID3v2.frame_id_of_string v
 
 let id3v2_of_metadata ~version m =
   let frames =
@@ -609,7 +506,9 @@ let id3v2_of_metadata ~version m =
           | Some id ->
               {
                 Metadata.ID3v2.id;
-                data = `Text (`UTF_8, v);
+                data =
+                  (if Metadata.ID3v2.binary_frame id then `Binary v
+                   else `Text (`UTF_8, v));
                 flags = Metadata.ID3v2.default_flags id;
               }
               :: frames
@@ -617,3 +516,8 @@ let id3v2_of_metadata ~version m =
       [] m
   in
   Metadata.ID3v2.make ~version frames
+
+let is_docker =
+  Lazy.from_fun (fun () ->
+      Sys.unix
+      && Sys.command "grep 'docker\\|lxc' /proc/1/cgroup >/dev/null 2>&1" = 0)
