@@ -1,6 +1,6 @@
 open Liquidsoap_lang
 
-let deprecated_features = [("GStreamer", Gstreamer_option.enabled)]
+let deprecated_features = []
 
 let build_config =
   let path_mode =
@@ -44,6 +44,8 @@ let build_config =
    - scripted binaries : %{Liquidsoap_paths.bin_dir_descr}
    - rundir            : %{Liquidsoap_paths.rundir_descr}
    - logdir            : %{Liquidsoap_paths.logdir_descr}
+   - user cache        : %{Liquidsoap_paths.user_cache_override_descr} (override with $LIQ_CACHE_USER_DIR)
+   - system cache      : %{Liquidsoap_paths.system_cache_override_descr} (override with $LIQ_CACHE_SYSTEM_DIR)
    - camomile files    : %{Liquidsoap_paths.camomile_dir_descr}
 
  * Supported input formats
@@ -56,10 +58,11 @@ let build_config =
    - Speex             : %{Speex_option.detected}
    - Theora            : %{Theora_option.detected}
    - Vorbis            : %{Vorbis_option.detected}
+   - WAV/AIFF          : yes (native)
 
  * Supported output formats
    - FDK-AAC           : %{Fdkaac_option.detected}
-   - Ffmpeg            : %{Ffmpeg_option.detected}
+   - FFmpeg            : %{Ffmpeg_option.detected}
    - MP3               : %{Lame_option.detected}
    - MP3 (fixed-point) : %{Shine_option.detected}
    - Flac (native)     : %{Flac_option.detected}
@@ -68,17 +71,22 @@ let build_config =
    - Speex             : %{Speex_option.detected}
    - Theora            : %{Theora_option.detected}
    - Vorbis            : %{Vorbis_option.detected}
+   - WAV/AIFF          : yes (native)
 
  * Tags
-   - Taglib (ID3 tags) : %{Taglib_option.detected}
+   - AAC               : %{Faad_option.detected}
+   - FFmpeg            : %{Ffmpeg_option.detected}
+   - FLAC (native)     : %{Flac_option.detected}
+   - Flac (ogg)        : %{Ogg_flac_option.detected}
+   - Native decoder    : yes
    - Vorbis            : %{Vorbis_option.detected}
 
  * Input / output
    - ALSA              : %{Alsa_option.detected}
    - AO                : %{Ao_option.detected}
    - FFmpeg            : %{Ffmpeg_option.detected}
-   - GStreamer         : %{Gstreamer_option.detected}
    - JACK              : %{Bjack_option.detected}
+   - NDI               : %{Ndi_option.detected}
    - OSS               : %{Oss_option.detected}
    - Portaudio         : %{Portaudio_option.detected}
    - Pulseaudio        : %{Pulseaudio_option.detected}
@@ -116,9 +124,9 @@ let build_config =
    - lastfm            : %{Lastfm_option.detected}
    - lo                : %{Lo_option.detected}
    - memtrace          : %{Memtrace_option.detected}
-   - mem_usage         : %{Mem_usage_option.detected}
    - osc               : %{Osc_option.detected}
    - ssl               : %{Ssl_option.detected}
+   - sqlite3           : %{Sqlite3_option.detected}
    - tls               : %{Tls_option.detected}
    - posix-time2       : %{Posix_time_option.detected}
    - windows service   : %{Winsvc_option.detected}
@@ -135,7 +143,6 @@ let opam_config =
 opam-version: "2.0"
 variables {
   ffmpeg-enabled: %{string_of_bool Ffmpeg_option.enabled}
-  gstreamer-enabled: %{string_of_bool Gstreamer_option.enabled}
   ssl-enabled: %{string_of_bool Ssl_option.enabled}
   tls-enabled: %{string_of_bool Tls_option.enabled}
 }
